@@ -5,10 +5,11 @@ use version_ranges::Ranges;
 use uv_distribution_filename::WheelFilename;
 use uv_pep440::{
     LowerBound, UpperBound, Version, VersionSpecifier, VersionSpecifiers,
-    release_specifiers_to_ranges,
+    release_specifier_to_range, release_specifiers_to_ranges,
 };
 use uv_pep508::{MarkerExpression, MarkerTree, MarkerValueVersion};
 use uv_platform_tags::{AbiTag, LanguageTag};
+use uv_warnings::warn_user_once;
 
 /// The `Requires-Python` requirement specifier.
 ///
@@ -66,15 +67,28 @@ impl RequiresPython {
     ) -> Option<Self> {
         // Convert to PubGrub range and perform an intersection.
         let range = specifiers
-            .into_iter()
-            .map(|specifier| release_specifiers_to_ranges(specifier.clone()))
-            .fold(None, |range: Option<Ranges<Version>>, requires_python| {
-                if let Some(range) = range {
-                    Some(range.intersection(&requires_python))
-                } else {
-                    Some(requires_python)
+            .map(|specs| {
+                // Warn if there’s exactly one `~=` specifier without a patch.
+                if let [spec] = &specs[..] {
+                    if spec.is_tilde_without_patch() {
+                        if let Some((lo_b, hi_b)) = release_specifier_to_range(spec.clone())
+                            .bounding_range()
+                            .map(|(l, u)| (l.cloned(), u.cloned()))
+                        {
+                            let lo_spec = LowerBound::new(lo_b).specifier().unwrap();
+                            let hi_spec = UpperBound::new(hi_b).specifier().unwrap();
+                            warn_user_once!(
+                                "The release specifier (`{spec}`) contains a compatible release \
+                                match without a patch version. This will be interpreted as \
+                                `{lo_spec}, {hi_spec}`. Did you mean `{spec}.0` to freeze the minor \
+                                version?"
+                            );
+                        }
+                    }
                 }
-            })?;
+                release_specifiers_to_ranges(specs.clone())
+            })
+            .reduce(|acc, r| acc.intersection(&r))?;
 
         // If the intersection is empty, return `None`.
         if range.is_empty() {
