@@ -12,7 +12,6 @@ use itertools::Itertools;
 use reqwest::header::AUTHORIZATION;
 use reqwest::multipart::Part;
 use reqwest::{Body, Response, StatusCode};
-use reqwest_middleware::RequestBuilder;
 use reqwest_retry::policies::ExponentialBackoff;
 use reqwest_retry::{RetryPolicy, Retryable, RetryableStrategy};
 use rustc_hash::FxHashSet;
@@ -29,7 +28,7 @@ use uv_auth::Credentials;
 use uv_cache::{Cache, Refresh};
 use uv_client::{
     BaseClient, DEFAULT_RETRIES, MetadataFormat, OwnedArchive, RegistryClientBuilder,
-    UvRetryableStrategy,
+    RequestBuilder, UvRetryableStrategy,
 };
 use uv_configuration::{KeyringProviderType, TrustedPublishing};
 use uv_distribution_filename::{DistFilename, SourceDistExtension, SourceDistFilename};
@@ -330,7 +329,9 @@ pub async fn check_trusted_publishing(
             debug!(
                 "Running on GitHub Actions without explicit credentials, checking for trusted publishing"
             );
-            match trusted_publishing::get_token(registry, client.for_host(registry)).await {
+            match trusted_publishing::get_token(registry, client.for_host(registry).raw_client())
+                .await
+            {
                 Ok(token) => Ok(TrustedPublishResult::Configured(token)),
                 Err(err) => {
                     // TODO(konsti): It would be useful if we could differentiate between actual errors
@@ -364,7 +365,9 @@ pub async fn check_trusted_publishing(
                 );
             }
 
-            let token = trusted_publishing::get_token(registry, client.for_host(registry)).await?;
+            let token =
+                trusted_publishing::get_token(registry, client.for_host(registry).raw_client())
+                    .await?;
             Ok(TrustedPublishResult::Configured(token))
         }
         TrustedPublishing::Never => Ok(TrustedPublishResult::Skipped),
@@ -387,7 +390,7 @@ pub async fn upload(
     download_concurrency: &Semaphore,
     reporter: Arc<impl Reporter>,
 ) -> Result<bool, PublishError> {
-    let form_metadata = form_metadata(file, filename)
+    let form_metadata = FormMetadata::read_from_file(file, filename)
         .await
         .map_err(|err| PublishError::PublishPrepare(file.to_path_buf(), Box::new(err)))?;
 
@@ -641,125 +644,135 @@ async fn metadata(file: &Path, filename: &DistFilename) -> Result<Metadata23, Pu
     Ok(Metadata23::parse(&contents)?)
 }
 
-/// Collect the non-file fields for the multipart request from the package METADATA.
-///
-/// Reference implementation: <https://github.com/pypi/warehouse/blob/d2c36d992cf9168e0518201d998b2707a3ef1e72/warehouse/forklift/legacy.py#L1376-L1430>
-async fn form_metadata(
-    file: &Path,
-    filename: &DistFilename,
-) -> Result<Vec<(&'static str, String)>, PublishPrepareError> {
-    let hash_hex = hash_file(file, Hasher::from(HashAlgorithm::Sha256)).await?;
+#[derive(Debug, Clone)]
+struct FormMetadata(Vec<(&'static str, String)>);
 
-    let Metadata23 {
-        metadata_version,
-        name,
-        version,
-        platforms,
-        // Not used by PyPI legacy upload
-        supported_platforms: _,
-        summary,
-        description,
-        description_content_type,
-        keywords,
-        home_page,
-        download_url,
-        author,
-        author_email,
-        maintainer,
-        maintainer_email,
-        license,
-        license_expression,
-        license_files,
-        classifiers,
-        requires_dist,
-        provides_dist,
-        obsoletes_dist,
-        requires_python,
-        requires_external,
-        project_urls,
-        provides_extras,
-        dynamic,
-    } = metadata(file, filename).await?;
+impl FormMetadata {
+    /// Collect the non-file fields for the multipart request from the package METADATA.
+    ///
+    /// Reference implementation: <https://github.com/pypi/warehouse/blob/d2c36d992cf9168e0518201d998b2707a3ef1e72/warehouse/forklift/legacy.py#L1376-L1430>
+    async fn read_from_file(
+        file: &Path,
+        filename: &DistFilename,
+    ) -> Result<Self, PublishPrepareError> {
+        let hash_hex = hash_file(file, Hasher::from(HashAlgorithm::Sha256)).await?;
 
-    let mut form_metadata = vec![
-        (":action", "file_upload".to_string()),
-        ("sha256_digest", hash_hex.digest.to_string()),
-        ("protocol_version", "1".to_string()),
-        ("metadata_version", metadata_version.clone()),
-        // Twine transforms the name with `re.sub("[^A-Za-z0-9.]+", "-", name)`
-        // * <https://github.com/pypa/twine/issues/743>
-        // * <https://github.com/pypa/twine/blob/5bf3f38ff3d8b2de47b7baa7b652c697d7a64776/twine/package.py#L57-L65>
-        // warehouse seems to call `packaging.utils.canonicalize_name` nowadays and has a separate
-        // `normalized_name`, so we'll start with this and we'll readjust if there are user reports.
-        ("name", name.clone()),
-        ("version", version.clone()),
-        ("filetype", filename.filetype().to_string()),
-    ];
+        let Metadata23 {
+            metadata_version,
+            name,
+            version,
+            platforms,
+            // Not used by PyPI legacy upload
+            supported_platforms: _,
+            summary,
+            description,
+            description_content_type,
+            keywords,
+            home_page,
+            download_url,
+            author,
+            author_email,
+            maintainer,
+            maintainer_email,
+            license,
+            license_expression,
+            license_files,
+            classifiers,
+            requires_dist,
+            provides_dist,
+            obsoletes_dist,
+            requires_python,
+            requires_external,
+            project_urls,
+            provides_extras,
+            dynamic,
+        } = metadata(file, filename).await?;
 
-    if let DistFilename::WheelFilename(wheel) = filename {
-        form_metadata.push(("pyversion", wheel.python_tags().iter().join(".")));
-    } else {
-        form_metadata.push(("pyversion", "source".to_string()));
+        let mut form_metadata = vec![
+            (":action", "file_upload".to_string()),
+            ("sha256_digest", hash_hex.digest.to_string()),
+            ("protocol_version", "1".to_string()),
+            ("metadata_version", metadata_version.clone()),
+            // Twine transforms the name with `re.sub("[^A-Za-z0-9.]+", "-", name)`
+            // * <https://github.com/pypa/twine/issues/743>
+            // * <https://github.com/pypa/twine/blob/5bf3f38ff3d8b2de47b7baa7b652c697d7a64776/twine/package.py#L57-L65>
+            // warehouse seems to call `packaging.utils.canonicalize_name` nowadays and has a separate
+            // `normalized_name`, so we'll start with this and we'll readjust if there are user reports.
+            ("name", name.clone()),
+            ("version", version.clone()),
+            ("filetype", filename.filetype().to_string()),
+        ];
+
+        if let DistFilename::WheelFilename(wheel) = filename {
+            form_metadata.push(("pyversion", wheel.python_tags().iter().join(".")));
+        } else {
+            form_metadata.push(("pyversion", "source".to_string()));
+        }
+
+        let mut add_option = |name, value: Option<String>| {
+            if let Some(some) = value.clone() {
+                form_metadata.push((name, some));
+            }
+        };
+
+        add_option("author", author);
+        add_option("author_email", author_email);
+        add_option("description", description);
+        add_option("description_content_type", description_content_type);
+        add_option("download_url", download_url);
+        add_option("home_page", home_page);
+        add_option("keywords", keywords);
+        add_option("license", license);
+        add_option("license_expression", license_expression);
+        add_option("maintainer", maintainer);
+        add_option("maintainer_email", maintainer_email);
+        add_option("summary", summary);
+
+        // The GitLab PyPI repository API implementation requires this metadata field and twine always
+        // includes it in the request, even when it's empty.
+        form_metadata.push(("requires_python", requires_python.unwrap_or(String::new())));
+
+        let mut add_vec = |name, values: Vec<String>| {
+            for i in values {
+                form_metadata.push((name, i.clone()));
+            }
+        };
+
+        add_vec("classifiers", classifiers);
+        add_vec("dynamic", dynamic);
+        add_vec("license_file", license_files);
+        add_vec("obsoletes_dist", obsoletes_dist);
+        add_vec("platform", platforms);
+        add_vec("project_urls", project_urls);
+        add_vec("provides_dist", provides_dist);
+        add_vec("provides_extra", provides_extras);
+        add_vec("requires_dist", requires_dist);
+        add_vec("requires_external", requires_external);
+
+        Ok(Self(form_metadata))
     }
 
-    let mut add_option = |name, value: Option<String>| {
-        if let Some(some) = value.clone() {
-            form_metadata.push((name, some));
-        }
-    };
-
-    add_option("author", author);
-    add_option("author_email", author_email);
-    add_option("description", description);
-    add_option("description_content_type", description_content_type);
-    add_option("download_url", download_url);
-    add_option("home_page", home_page);
-    add_option("keywords", keywords);
-    add_option("license", license);
-    add_option("license_expression", license_expression);
-    add_option("maintainer", maintainer);
-    add_option("maintainer_email", maintainer_email);
-    add_option("summary", summary);
-
-    // The GitLab PyPI repository API implementation requires this metadata field and twine always
-    // includes it in the request, even when it's empty.
-    form_metadata.push(("requires_python", requires_python.unwrap_or(String::new())));
-
-    let mut add_vec = |name, values: Vec<String>| {
-        for i in values {
-            form_metadata.push((name, i.clone()));
-        }
-    };
-
-    add_vec("classifiers", classifiers);
-    add_vec("dynamic", dynamic);
-    add_vec("license_file", license_files);
-    add_vec("obsoletes_dist", obsoletes_dist);
-    add_vec("platform", platforms);
-    add_vec("project_urls", project_urls);
-    add_vec("provides_dist", provides_dist);
-    add_vec("provides_extra", provides_extras);
-    add_vec("requires_dist", requires_dist);
-    add_vec("requires_external", requires_external);
-
-    Ok(form_metadata)
+    /// Returns an iterator over the metadata fields.
+    fn iter(&self) -> std::slice::Iter<'_, (&'static str, String)> {
+        self.0.iter()
+    }
 }
 
 /// Build the upload request.
 ///
 /// Returns the request and the reporter progress bar id.
-async fn build_request(
+async fn build_request<'a>(
     file: &Path,
     raw_filename: &str,
     filename: &DistFilename,
     registry: &DisplaySafeUrl,
-    client: &BaseClient,
+    client: &'a BaseClient,
     credentials: &Credentials,
-    form_metadata: &[(&'static str, String)],
+    form_metadata: &FormMetadata,
     reporter: Arc<impl Reporter>,
-) -> Result<(RequestBuilder, usize), PublishPrepareError> {
+) -> Result<(RequestBuilder<'a>, usize), PublishPrepareError> {
     let mut form = reqwest::multipart::Form::new();
-    for (key, value) in form_metadata {
+    for (key, value) in form_metadata.iter() {
         form = form.text(*key, value.clone());
     }
 
@@ -885,16 +898,19 @@ async fn handle_response(registry: &Url, response: Response) -> Result<(), Publi
 
 #[cfg(test)]
 mod tests {
-    use crate::{Reporter, build_request, form_metadata};
-    use insta::{assert_debug_snapshot, assert_snapshot};
-    use itertools::Itertools;
     use std::path::PathBuf;
     use std::sync::Arc;
+
+    use insta::{assert_debug_snapshot, assert_snapshot};
+    use itertools::Itertools;
+
     use uv_auth::Credentials;
     use uv_client::BaseClientBuilder;
     use uv_distribution_filename::DistFilename;
     use uv_redacted::DisplaySafeUrl;
 
+    use crate::{FormMetadata, Reporter, build_request};
+
     struct DummyReporter;
 
     impl Reporter for DummyReporter {
@@ -913,7 +929,9 @@ mod tests {
         let file = PathBuf::from("../../scripts/links/").join(raw_filename);
         let filename = DistFilename::try_from_normalized_filename(raw_filename).unwrap();
 
-        let form_metadata = form_metadata(&file, &filename).await.unwrap();
+        let form_metadata = FormMetadata::read_from_file(&file, &filename)
+            .await
+            .unwrap();
 
         let formatted_metadata = form_metadata
             .iter()
@@ -969,12 +987,13 @@ mod tests {
         project_urls: Source, https://github.com/unknown/tqdm
         "###);
 
+        let client = BaseClientBuilder::new().build();
         let (request, _) = build_request(
             &file,
             raw_filename,
             &filename,
             &DisplaySafeUrl::parse("https://example.org/upload").unwrap(),
-            &BaseClientBuilder::new().build(),
+            &client,
             &Credentials::basic(Some("ferris".to_string()), Some("F3RR!S".to_string())),
             &form_metadata,
             Arc::new(DummyReporter),
@@ -985,7 +1004,7 @@ mod tests {
         insta::with_settings!({
             filters => [("boundary=[0-9a-f-]+", "boundary=[...]")],
         }, {
-            assert_debug_snapshot!(&request, @r#"
+            assert_debug_snapshot!(&request.raw_builder(), @r#"
             RequestBuilder {
                 inner: RequestBuilder {
                     method: POST,
@@ -1024,7 +1043,9 @@ mod tests {
         let file = PathBuf::from("../../scripts/links/").join(raw_filename);
         let filename = DistFilename::try_from_normalized_filename(raw_filename).unwrap();
 
-        let form_metadata = form_metadata(&file, &filename).await.unwrap();
+        let form_metadata = FormMetadata::read_from_file(&file, &filename)
+            .await
+            .unwrap();
 
         let formatted_metadata = form_metadata
             .iter()
@@ -1118,12 +1139,13 @@ mod tests {
         requires_dist: requests ; extra == 'telegram'
         "###);
 
+        let client = BaseClientBuilder::new().build();
         let (request, _) = build_request(
             &file,
             raw_filename,
             &filename,
             &DisplaySafeUrl::parse("https://example.org/upload").unwrap(),
-            &BaseClientBuilder::new().build(),
+            &client,
             &Credentials::basic(Some("ferris".to_string()), Some("F3RR!S".to_string())),
             &form_metadata,
             Arc::new(DummyReporter),
@@ -1134,7 +1156,7 @@ mod tests {
         insta::with_settings!({
             filters => [("boundary=[0-9a-f-]+", "boundary=[...]")],
         }, {
-            assert_debug_snapshot!(&request, @r#"
+            assert_debug_snapshot!(&request.raw_builder(), @r#"
             RequestBuilder {
                 inner: RequestBuilder {
                     method: POST,
