@@ -7,7 +7,7 @@ use tracing::debug;
 
 use uv_cache::Cache;
 use uv_client::BaseClientBuilder;
-use uv_configuration::{DryRun, KeyringProviderType};
+use uv_configuration::{DryRun, KeyringProviderType, PreviewMode};
 use uv_distribution_types::Requirement;
 use uv_distribution_types::{InstalledMetadata, Name, UnresolvedRequirement};
 use uv_fs::Simplified;
@@ -19,7 +19,7 @@ use uv_python::{Prefix, PythonEnvironment, Target};
 use uv_requirements::{RequirementsSource, RequirementsSpecification};
 
 use crate::commands::pip::operations::report_target_environment;
-use crate::commands::{elapsed, ExitStatus};
+use crate::commands::{ExitStatus, elapsed};
 use crate::printer::Printer;
 use crate::settings::NetworkSettings;
 
@@ -37,6 +37,7 @@ pub(crate) async fn pip_uninstall(
     network_settings: &NetworkSettings,
     dry_run: DryRun,
     printer: Printer,
+    preview: PreviewMode,
 ) -> Result<ExitStatus> {
     let start = std::time::Instant::now();
 
@@ -57,6 +58,7 @@ pub(crate) async fn pip_uninstall(
             .unwrap_or_default(),
         EnvironmentPreference::from_system_flag(system, true),
         &cache,
+        preview,
     )?;
 
     report_target_environment(&environment, &cache, printer)?;
