@@ -29,7 +29,7 @@ use crate::commands::project::{
 };
 use crate::commands::reporters::PythonDownloadReporter;
 use crate::commands::tool::common::remove_entrypoints;
-use crate::commands::{ExitStatus, conjunction, tool::common::install_executables};
+use crate::commands::{ExitStatus, conjunction, tool::common::finalize_tool_install};
 use crate::printer::Printer;
 use crate::settings::{NetworkSettings, ResolverInstallerSettings};
 
@@ -99,6 +99,7 @@ pub(crate) async fn upgrade(
                 install_mirrors.python_install_mirror.as_deref(),
                 install_mirrors.pypy_install_mirror.as_deref(),
                 install_mirrors.python_downloads_json_url.as_deref(),
+                preview,
             )
             .await?
             .into_interpreter(),
@@ -308,7 +309,7 @@ async fn upgrade_tool(
         )
         .await?;
 
-        let environment = installed_tools.create_environment(name, interpreter.clone())?;
+        let environment = installed_tools.create_environment(name, interpreter.clone(), preview)?;
 
         let environment = sync_environment(
             environment,
@@ -375,7 +376,7 @@ async fn upgrade_tool(
         remove_entrypoints(&existing_tool_receipt);
 
         // If we modified the target tool, reinstall the entrypoints.
-        install_executables(
+        finalize_tool_install(
             &environment,
             name,
             installed_tools,
